@@ -2,10 +2,10 @@
  * libdvdcss.h: private DVD reading library data
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: libdvdcss.h 152 2004-02-24 15:47:00Z sam $
+ * $Id$
  *
- * Authors: Stphane Borel <stef@via.ecp.fr>
- *          Samuel Hocevar <sam@zoy.org>
+ * Authors: Stéphane Borel <stef@via.ecp.fr>
+ *          Sam Hocevar <sam@zoy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,6 +78,19 @@ struct dvdcss_s
  * Functions used across the library
  *****************************************************************************/
 #define print_error(dvdcss,msg) _print_error(dvdcss,msg)
+#if defined( _MSC_VER )
+#include <stdarg.h>
+__forceinline void print_debug(dvdcss_t dvdcss, const char *msg,...)
+{
+	va_list args;
+
+    fprintf( stderr, "libdvdcss debug: " );
+	va_start( args, msg );
+    vfprintf( stderr, msg, args );
+	va_end( args );
+    fprintf( stderr, "\n" );
+}
+#else
 #define print_debug(dvdcss,msg,args...) \
     if( dvdcss->b_debug ) \
     { \
@@ -85,6 +98,7 @@ struct dvdcss_s
         fprintf( stderr, msg, ##args ); \
         fprintf( stderr, "\n" ); \
     }
+#endif
 
 void _print_error ( dvdcss_t, char * );
 
